# Copyright (c) 2016, 2017, 2018, 2019, 2020, Sqreen. All rights reserved.
# Please refer to our terms for more information:
#
#     https://www.sqreen.io/terms.html
#

# The lib version is always contained at the beginning of __version__
__version__ = "1.0.4.0.0"
__lib_version__ = "1.0.4"
